"use strict";
// Copyright (c) Frédéric Maquin <fred@ephread.com>
// Licensed under the MIT License.
// See LICENSE in the project root for license information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const inklecate_1 = require("../../inklecate");
const identifiers_1 = require("../../types/identifiers");
const StoryRenderer_1 = require("./StoryRenderer");
/**
 * Runs command sent by the client.
 */
class CommandRunner {
    constructor(connection, diagnosticManager, workspaceManager, logger) {
        this.connection = connection;
        this.diagnosticManager = diagnosticManager;
        this.workspaceManager = workspaceManager;
        this.logger = logger;
    }
    /**
     * Compile the story to which the given URI belong.
     *
     * @param params compile parameters, containing a file URI.
     */
    compileStory(params) {
        this.logger.console.info("Received compilation command.");
        this.workspaceManager.getDocumentPathFromParams(params).then(pathAndWorkspace => {
            this.executeCompileCommand(pathAndWorkspace.documentPath, pathAndWorkspace.workspace);
        }, errorMessage => {
            const message = `The project could not be compiled: ${errorMessage}`;
            this.logger.showErrorMessage(message, true);
        });
        return true;
    }
    /**
     * Run the story to which the given URI belong.
     *
     * @param params compile parameters, containing a file URI.
     */
    playStory(params) {
        this.logger.console.info("Received play story command.");
        this.workspaceManager.getDocumentPathFromParams(params).then(pathAndWorkspace => {
            this.executeCompileCommand(pathAndWorkspace.documentPath, pathAndWorkspace.workspace, true);
        }, errorMessage => {
            const message = `The project could not be compiled: ${errorMessage}`;
            this.logger.showErrorMessage(message, true);
        });
        return true;
    }
    /**
     * If a story is currently playing and prompting for a choice selection,
     * select the choice at the given index.
     *
     * @param params parameters, containing the index of the chocie to select.
     */
    selectOption(params) {
        this.logger.console.info("Received select option command.");
        if (!params.arguments || !params.arguments[0]) {
            this.logger.showErrorMessage(`${identifiers_1.Commands.selectOption} error: the command was called with no arguments.`, true);
            return;
        }
        const index = parseInt(params.arguments[0]);
        inklecate_1.chooseOption(index);
    }
    /**
     * Kill the current Inklecate process, if applicable.
     */
    killInklecate() {
        this.logger.console.info("Received kill inklecate command.");
        inklecate_1.killInklecate();
    }
    /**
     * Compile the story contained in `workspace`.
     * `documentPath` is required to retrieve the proper settings, which
     * are scoped to the resource.
     *
     * @param documentUri the document URI used to retrieved the settings.
     * @param workspace the ink workspace to compile.
     */
    executeCompileCommand(documentUri, workspace, play = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.workspaceManager.canCompile) {
                this.logger.console.info("The workspace is not ready yet, try compiling again in a few seconds…");
                this.logger.showInformationMessage("Ink support is still initializing, please try again in a few seconds.", false);
                return;
            }
            const settings = yield this.workspaceManager.fetchDocumentConfigurationSettings(documentUri);
            const storyRenderer = play ? new StoryRenderer_1.default(this.connection) : undefined;
            inklecate_1.compileProject(settings, workspace, this.logger, storyRenderer, this.diagnosticManager);
        });
    }
}
exports.default = CommandRunner;
//# sourceMappingURL=CommandRunner.js.map