"use strict";
// Copyright (c) Frédéric Maquin <fred@ephread.com>
// Licensed under the MIT License.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_uri_1 = require("vscode-uri");
const types_1 = require("../../types/types");
const utils_1 = require("../utils");
const identifiers_1 = require("../../types/identifiers");
/**
 * Manages the diagnostic reported by Inklecate.
 */
class DiagnosticManager {
    constructor(connection, documentManager, logger) {
        this.connection = connection;
        this.documentManager = documentManager;
        this.logger = logger;
    }
    /**
     * Send the given errors to the client.
     *
     * @param errors the errors to push.
     */
    notifyClientAndPushDiagnostics(workspace, outputStoryPath, errors) {
        for (const textDocument of this.documentManager.documents.all()) {
            const diagnostics = [];
            for (const error of errors) {
                if (vscode_uri_1.default.parse(textDocument.uri).fsPath === error.filePath) {
                    let message = error.message;
                    if (error.type === types_1.InkErrorType.Todo) {
                        message = `Todo: ${message}`;
                    }
                    const diagnostic = {
                        severity: utils_1.getDiagnosticSeverityFromInkErrorType(error.type),
                        range: {
                            start: { line: error.lineNumber - 1, character: 0 },
                            end: { line: error.lineNumber, character: 0 }
                        },
                        message,
                        source: "inklecate"
                    };
                    diagnostics.push(diagnostic);
                }
            }
            // If there a no errors to report, we'll send a custom notification to
            // the client, asking it to remember the path to the compiled story.
            if (diagnostics.length === 0) {
                const params = {
                    workspaceUri: workspace.folder.uri,
                    storyUri: `file://${outputStoryPath}`
                };
                this.connection.sendNotification(identifiers_1.CompilationNotification.didCompileStory, params);
            }
            this.connection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
        }
    }
}
exports.default = DiagnosticManager;
//# sourceMappingURL=DiagnosticManager.js.map