import { ClientCapabilities, DidChangeWatchedFilesParams, ExecuteCommandParams, TextDocument } from "vscode-languageserver";
import { IConnection } from "vscode-languageserver";
import { Capabilities, DocumentPathAndWorkspace, IConnectionLogger, InkWorkspace, PartialInkConfigurationSettings } from "../../types/types";
import DiagnosticManager from "./DiagnosticManager";
import DocumentManager from "./DocumentManager";
/**
 * Manages the local workspaces, mirroring the workspaces handled by the client.
 */
export default class WorkspaceManager {
    private connection;
    private documentManager;
    private diagnosticManager;
    private logger;
    /**
     * Workspace directories managed by the server,
     * mapped from the workspace managed by the client.
     *
     * URIs of the client's `WorkspaceFolder` will be used as keys.
     */
    workspaceDirectories: Map<string, InkWorkspace>;
    /** Capabilities defined by the client. Defaults to none. */
    capabilities: Capabilities;
    canCompile: boolean;
    constructor(connection: IConnection, documentManager: DocumentManager, diagnosticManager: DiagnosticManager, logger: IConnectionLogger);
    initializeCapabilities(clientCapabilities: ClientCapabilities): void;
    /**
     * Returns the InkWorkspace of the given document. If no workspace could be found,
     * returns `undefined`.
     *
     * Since Inklecate requires as a full compilation of the entire related project
     * to push diagnostics, the workspace of each document must be known.
     *
     * @param document a text document, belonging to the returned workspace.
     */
    getInkWorkspaceOfDocument(document: TextDocument): InkWorkspace | undefined;
    /**
     * Returns the InkWorkspace of the given uri. If no workspace could be
     * found, returns `undefined`.
     *
     * Since Inklecate requires as a full compilation of the entire related project
     * to push diagnostics, the workspace of each document must be known.
     *
     * @param document a text document, belonging to the returned workspace.
     */
    getInkWorkspaceOfFilePath(documentPath: string): InkWorkspace | undefined;
    /**
     * Initialize workspaces by fetching opened `WorkspaceFolder` from the client and
     * creating the temporary directories which will hold copies of the Ink Project.
     */
    initializeInkWorkspaces(): Promise<void>;
    /**
     * Update the content of the given document, in the temporary copy of the workspace,
     * and compile the project.
     *
     * @param document the document to update.
     */
    updateDocumentAndCompileWorkspace(document: TextDocument): Promise<void>;
    /**
     * Retrieve the created from params and copy them to the appropriate
     * temporary directory.
     *
     * @param params parameters sent by the client.
     */
    copyNewlyCreatedFiles(params: DidChangeWatchedFilesParams): Promise<void>;
    /**
     * Fetch the configuration settings of the given document, from the client.
     *
     * @param document the document of which fetch the configuration settings,
     *                 can either be a TextDocument or a string-based uri.
     */
    fetchDocumentConfigurationSettings(documentOrUri: TextDocument | string): Thenable<PartialInkConfigurationSettings>;
    getDocumentPathFromParams(params: ExecuteCommandParams): Promise<DocumentPathAndWorkspace>;
}
