"use strict";
// Copyright (c) Frédéric Maquin <fred@ephread.com>
// Licensed under the MIT License.
// See LICENSE in the project root for license information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = require("path");
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const inklecate_1 = require("../../inklecate");
const configuration_1 = require("../configuration");
const utils_1 = require("../utils");
/**
 * Manages the local workspaces, mirroring the workspaces handled by the client.
 */
class WorkspaceManager {
    constructor(connection, documentManager, diagnosticManager, logger) {
        this.connection = connection;
        this.documentManager = documentManager;
        this.diagnosticManager = diagnosticManager;
        this.logger = logger;
        /**
         * Workspace directories managed by the server,
         * mapped from the workspace managed by the client.
         *
         * URIs of the client's `WorkspaceFolder` will be used as keys.
         */
        this.workspaceDirectories = new Map();
        /** Capabilities defined by the client. Defaults to none. */
        this.capabilities = {
            configuration: false,
            workspaceFolder: false,
            diagnostic: false
        };
        this.canCompile = false;
    }
    initializeCapabilities(clientCapabilities) {
        if (clientCapabilities.workspace) {
            this.capabilities.configuration = !!clientCapabilities.workspace.configuration;
            this.capabilities.workspaceFolder = !!clientCapabilities.workspace.workspaceFolders;
            if (clientCapabilities.textDocument) {
                this.capabilities.diagnostic = !!clientCapabilities.textDocument.publishDiagnostics;
            }
        }
    }
    /**
     * Returns the InkWorkspace of the given document. If no workspace could be found,
     * returns `undefined`.
     *
     * Since Inklecate requires as a full compilation of the entire related project
     * to push diagnostics, the workspace of each document must be known.
     *
     * @param document a text document, belonging to the returned workspace.
     */
    getInkWorkspaceOfDocument(document) {
        const documentPath = vscode_uri_1.default.parse(document.uri).fsPath;
        return this.getInkWorkspaceOfFilePath(documentPath);
    }
    /**
     * Returns the InkWorkspace of the given uri. If no workspace could be
     * found, returns `undefined`.
     *
     * Since Inklecate requires as a full compilation of the entire related project
     * to push diagnostics, the workspace of each document must be known.
     *
     * @param document a text document, belonging to the returned workspace.
     */
    getInkWorkspaceOfFilePath(documentPath) {
        for (const workspaceKey of this.workspaceDirectories.keys()) {
            const workspace = this.workspaceDirectories.get(workspaceKey);
            if (!workspace) {
                continue;
            }
            const workspacePath = vscode_uri_1.default.parse(workspace.folder.uri).fsPath;
            if (utils_1.isFilePathChildOfDirPath(documentPath, workspacePath)) {
                return workspace;
            }
        }
        return undefined;
    }
    /**
     * Initialize workspaces by fetching opened `WorkspaceFolder` from the client and
     * creating the temporary directories which will hold copies of the Ink Project.
     */
    initializeInkWorkspaces() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.connection.workspace.getWorkspaceFolders().then(workspaceFolders => {
                if (workspaceFolders) {
                    const hrstart = process.hrtime();
                    const promises = [];
                    for (const workspaceFolder of workspaceFolders) {
                        promises.push(inklecate_1.prepareTempDirectoryForCompilation(workspaceFolder, this.logger)
                            .then(tempDir => {
                            if (tempDir) {
                                this.logger.console.info(`Temporary compilation directory successfully created at: ${tempDir}`);
                                let workspace = this.workspaceDirectories.get(workspaceFolder.uri);
                                if (!workspace) {
                                    workspace = {
                                        folder: workspaceFolder
                                    };
                                    this.workspaceDirectories.set(workspaceFolder.uri, workspace);
                                }
                                workspace.temporaryCompilationDirectory = tempDir;
                                this.canCompile = true;
                            }
                            else {
                                this.logger.console.error(`'tempDir' should have been set, but was undefined.`);
                                this.logger.reportServerError();
                            }
                        })
                            .catch(() => {
                            this.logger.reportServerError();
                        }));
                    }
                    Promise.all(promises).then(() => {
                        const hrend = process.hrtime(hrstart);
                        const time = (hrend[0] * 1e9 + hrend[1]) / 1e9;
                        this.logger.console.info(`All temporary compilation directories were created in ${time}s.`);
                    });
                }
            }, () => {
                return Promise.reject();
            });
        });
    }
    /**
     * Update the content of the given document, in the temporary copy of the workspace,
     * and compile the project.
     *
     * @param document the document to update.
     */
    updateDocumentAndCompileWorkspace(document) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.canCompile) {
                this.logger.console.info("The workspace is not ready yet, try compiling again in a few seconds…");
                return;
            }
            let workspace = this.getInkWorkspaceOfDocument(document);
            if (!workspace) {
                this.logger.console.warn("The temporary workspace does not exist, attempting to restore…");
                yield this.initializeInkWorkspaces();
                workspace = this.getInkWorkspaceOfDocument(document);
                if (!workspace) {
                    const basename = Path.basename(document.uri);
                    const message = `The temporary workspace is missing or ${basename} is not in the workspace.`;
                    this.logger.console.error(message);
                    return;
                }
            }
            const settings = yield this.fetchDocumentConfigurationSettings(document);
            inklecate_1.updateFile(document, workspace, this.logger, error => {
                if (error) {
                    this.logger.console.error(`Could not update '${document.uri}', ${error.message}`);
                    this.logger.reportServerError();
                }
                else {
                    inklecate_1.compileProject(settings, workspace, this.logger, undefined, this.diagnosticManager);
                }
            });
        });
    }
    /**
     * Retrieve the created from params and copy them to the appropriate
     * temporary directory.
     *
     * @param params parameters sent by the client.
     */
    copyNewlyCreatedFiles(params) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const change of params.changes) {
                if (change.type === vscode_languageserver_1.FileChangeType.Created) {
                    if (this.canCompile) {
                        const workspace = this.getInkWorkspaceOfFilePath(change.uri);
                        if (workspace) {
                            yield inklecate_1.copyNewlyCreatedFile(change.uri, workspace, this.logger);
                        }
                        else {
                            this.logger.console.error(`The temporary workspace is undefined, cannot copy newly created files.`);
                            this.logger.reportServerError();
                        }
                    }
                    else {
                        this.logger.console.warn("The workspace is not ready yet, ignoring newly created files");
                    }
                }
            }
        });
    }
    /**
     * Fetch the configuration settings of the given document, from the client.
     *
     * @param document the document of which fetch the configuration settings,
     *                 can either be a TextDocument or a string-based uri.
     */
    fetchDocumentConfigurationSettings(documentOrUri) {
        let documentUri;
        if (typeof documentOrUri === "string") {
            documentUri = documentOrUri;
        }
        else {
            documentUri = documentOrUri.uri;
        }
        if (!this.capabilities.configuration) {
            return Promise.resolve(Object.assign({}, configuration_1.getDefaultSettings()));
        }
        let result = this.documentManager.documentSettings.get(documentUri);
        if (!result) {
            result = this.connection.workspace.getConfiguration({
                scopeUri: documentUri,
                section: "ink"
            });
            this.documentManager.documentSettings.set(documentUri, result);
        }
        return result;
    }
    getDocumentPathFromParams(params) {
        return __awaiter(this, void 0, void 0, function* () {
            let fileURI;
            if (!params.arguments || params.arguments.length < 1) {
                fileURI = configuration_1.getDefaultSettings().mainStoryPath;
            }
            else {
                if (typeof params.arguments[0] !== "string") {
                    this.logger.console.warn("The file URI provided is not a string, the behavior might be undefined. " +
                        `The argument contained: ${JSON.stringify(params.arguments[0])}`);
                }
                fileURI = params.arguments[0];
            }
            const documentPath = vscode_uri_1.default.parse(fileURI).fsPath;
            const basename = Path.basename(documentPath);
            let workspace = this.getInkWorkspaceOfFilePath(documentPath);
            if (!workspace) {
                this.logger.console.warn("The temporary workspace does not exist, attempting to restore…");
                yield this.initializeInkWorkspaces();
                workspace = this.getInkWorkspaceOfFilePath(documentPath);
                if (!workspace) {
                    this.logger.console.error("The temporary workspace is still missing, aborting command.");
                    const message = `The temporary workspace is missing or ${basename} is not in the workspace.`;
                    return Promise.reject(message);
                }
            }
            return Promise.resolve({ documentPath, workspace });
        });
    }
}
exports.default = WorkspaceManager;
//# sourceMappingURL=WorkspaceManager.js.map