"use strict";
// Copyright (c) Frédéric Maquin <fred@ephread.com>
// Licensed under the MIT License.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const types_1 = require("./types/types");
const configuration_1 = require("./helpers/configuration");
const identifiers_1 = require("./types/identifiers");
const install_1 = require("./helpers/install");
const CommandRunner_1 = require("./helpers/Class/CommandRunner");
const DiagnosticManager_1 = require("./helpers/Class/DiagnosticManager");
const DocumentManager_1 = require("./helpers/Class/DocumentManager");
const WorkspaceManager_1 = require("./helpers/Class/WorkspaceManager");
/* Properties */
/******************************************************************************/
const connection = vscode_languageserver_1.createConnection(vscode_languageserver_1.ProposedFeatures.all);
const logger = createConnectionLogger(connection);
const documentManager = new DocumentManager_1.default();
const diagnosticManager = new DiagnosticManager_1.default(connection, documentManager, logger);
const workspaceManager = new WorkspaceManager_1.default(connection, documentManager, diagnosticManager, logger);
const commandRunner = new CommandRunner_1.default(connection, diagnosticManager, workspaceManager, logger);
/* Helpers */
/******************************************************************************/
/**
 * Create the logging wrapper.
 *
 * @param clientConnection the connection to which log.
 */
function createConnectionLogger(clientConnection) {
    const argv = process.argv.slice(2);
    const verbose = argv.indexOf('--verbose') > -1;
    return new types_1.InkConnectionLogger(clientConnection, verbose);
}
/* Connection callbacks */
/******************************************************************************/
connection.onInitialize((params) => {
    logger.console.info("Language Server connected.");
    workspaceManager.initializeCapabilities(params.capabilities);
    return {
        capabilities: {
            textDocumentSync: documentManager.documents.syncKind,
            executeCommandProvider: {
                commands: [
                    identifiers_1.Commands.compileStory,
                    identifiers_1.Commands.playStory,
                    identifiers_1.Commands.selectOption
                ]
            }
        }
    };
});
connection.onInitialized(() => {
    if (workspaceManager.capabilities.configuration) {
        connection.client.register(vscode_languageserver_1.DidChangeConfigurationNotification.type, undefined);
    }
    if (workspaceManager.capabilities.workspaceFolder) {
        connection.workspace.onDidChangeWorkspaceFolders(event => {
            workspaceManager.initializeInkWorkspaces();
        });
    }
    install_1.checkPlatformAndDownloadBinaryDependency(logger, (success) => {
        configuration_1.flagDefaultSettingsAsDirty();
        workspaceManager.initializeInkWorkspaces();
    });
});
connection.onDidChangeConfiguration(() => {
    documentManager.documentSettings.clear();
});
connection.onDidChangeWatchedFiles(workspaceManager.copyNewlyCreatedFiles);
connection.onExecuteCommand((params) => {
    switch (params.command) {
        case identifiers_1.Commands.compileStory:
            commandRunner.compileStory(params);
            break;
        case identifiers_1.Commands.playStory:
            commandRunner.playStory(params);
            break;
        case identifiers_1.Commands.selectOption:
            commandRunner.selectOption(params);
            break;
        default: break;
    }
});
/* Document callbacks */
/******************************************************************************/
documentManager.documents.onDidChangeContent(change => {
    workspaceManager.updateDocumentAndCompileWorkspace(change.document);
});
documentManager.documents.onDidClose(event => {
    documentManager.documentSettings.delete(event.document.uri);
});
/* Final setup */
/******************************************************************************/
documentManager.documents.listen(connection);
connection.listen();
//# sourceMappingURL=server.js.map